;;; org.el --- Outline-based notes management and organizer -*- lexical-binding: t; -*-

;; Carstens outline-mode for keeping track of everything.
;; Copyright (C) 2017 Free Software Foundation, Inc.
;;
;; Author: Rasmus Pank Roulund <emacs at pank dot eu>
;; Keywords: outlines, hypermedia, calendar, wp
;; Homepage: http://orgmode.org
;;
;; This file is part of GNU Emacs.
;;
;; GNU Emacs is free software: you can redistribute it and/or modify
;; it under the terms of the GNU General Public License as published by
;; the Free Software Foundation, either version 3 of the License, or
;; (at your option) any later version.

;; GNU Emacs is distributed in the hope that it will be useful,
;; but WITHOUT ANY WARRANTY; without even the implied warranty of
;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;; GNU General Public License for more details.

;; You should have received a copy of the GNU General Public License
;; along with GNU Emacs.  If not, see <https://www.gnu.org/licenses/>.
;;
;;; Commentary:
;;
;;; Code:

(require 'tempo)
(require 'cl-lib)

(defvar org-tempo-tags nil
  "Tempo tags for org-mode")

(defcustom org-structure-template-alist-keywords
  '((?L . "latex")
    (?H . "html")
    (?A . "ascii")
    (?i . "index"))
  "Keyword templates like `org-structure-template-alist' expanded via ."
  :group 'org-tempo
  :type '(repeat
	  (cons (character :tag "Key")
		(string :tag "Template")))
  :package-version '(Org . "9.2"))

(defun org-tempo-setup ()
  (tempo-use-tag-list 'org-tempo-tags)
  (setq-local tempo-match-finder "^ *\\(<[[:word:]]\\)\\="))

(add-hook 'org-mode-hook 'org-tempo-setup)


(defun org-tempo-add-templates ()
  "Update all org-tempo templates.

Goes through `org-structure-template-alist' and
`org-structure-template-alist-keywords'."
  (let ((keys (mapcar (apply-partially 'format "<%c")
		      (mapcar 'car (append org-structure-template-alist
					   org-structure-template-alist-keywords)))))
    (if (> (length keys)
           (length (delete-dups keys)))
	(user-error "Duplicated keys in `org-structure-template-alist' and `org-structure-template-alist-keywords'"))
    (mapcar (lambda (key)
	      (if (assoc-string key org-tempo-tags)
		  (setq org-tempo-tags
			(delete (assoc-string key org-tempo-tags)
				org-tempo-tags))))
	    keys)
    (mapcar 'org-tempo-add-block org-structure-template-alist)
    (mapcar 'org-tempo-add-keyword org-structure-template-alist-keywords))
  (setq tempo-dirty-collection t))

(defun org-tempo-add-block (entry)
  "Add block entry from `org-structure-template-alist'."
  (let* ((key (format "<%c" (car entry)))
	 (name (cdr entry)))
    (tempo-define-template (format "org-%s" (replace-regexp-in-string " " "-" name))
			   `(,(format "#+begin_%s " name) p '> n n
			     ,(format "#+end_%s" (car (org-split-string name " ")))
			     >)
			   key
			   (format "Insert a %s block" name)
			   'org-tempo-tags)))

(defun org-tempo-add-keyword (entry)
  "Add keyword entry from `org-structure-template-alist-keywords'."
  (let* ((key (format "<%c" (car entry)))
	 (name (cdr entry)))
    (tempo-define-template (format "org-%s" (replace-regexp-in-string " " "-" name))
			   `(,(format "#+%s: " name) p '>)
			   key
			   (format "Insert a %s keyword" name)
			   'org-tempo-tags)))
;;; Additional keywords

(tempo-define-template "org-include"
		       '("#+include: "
			 (ignore-errors
			   (format "\"%s\" " (file-relative-name (read-file-name "Include file: "))))
			 p >)
		       "<I"
		       "Include keyword"
		       'org-tempo-tags)

(with-eval-after-load 'org
  (org-tempo-add-templates)
  (add-hook 'org-tab-before-tab-emulation-hook
	    'tempo-complete-tag))

(provide 'org-tempo)
;;; org-tempo.el ends here
